%%% Canonical Structural Realization (CSR)
%%%
%%% (c) 1991, 1992, 1993  Sandiway Fong, NEC Research Institute, Inc.
%%%

%%% EXPORTS
%%%	csr(Role,C)			Canonical Structure Realization (CSR)
%%%	csrCheck(Role,C,P)
%%%	icsrCheck(Role,C,P)
%%%	ensureWhComp(Ci)		used in lexicon
%%%	optWhComp(Ci)			used in lexicon

csr(agent,np).
csr(benefactive,np).
csr(cognizer,np).
csr(experiencer,np).
csr(recipient,np).
csr(perceiver,np).
csr(beneficiary,np).
csr(patient,np).
csr(source,np).
csr(destination,np).
csr(theme,np).
csr(themeNA,np).
csr(themeA,np).
csr(instrument,np).
csr(state,np).
csr(location,np).
csr(possessor,np).
csr(goal,np).
csr(eq,np).
csr(eqObj,np).
csr(reducedProp,i2).				  % control verbs etc.
csr(propositionD,i2).				  % temporary
csr(propositionD,c2).
csr(proposition1,c2).
csr(proposition,c2).
csr(action,c2).
csr(proposition,np).
csr(pHolder(_),np). 
csr(predicate(X),X). 
csr(role(_),np).				  % short-hand

csr([Role],C) :- csr(Role,C).


csrRestriction(eqObj,goal((\+ ec(X)),X)).
csrRestriction(themeA,goal(intersectAGR(X,[[],[],[m,f]]),X)).
csrRestriction(themeNA,goal(intersectAGR(X,[[],[],n]),X)).
csrRestriction(possessor,goal(intersectAGR(X,[[],[],[m,f]]),X)).

csrCheck(proposition,np,X) :- 
	!,
	\+ X has_intrinsic_feature noCasemark(+).
csrCheck(_,_,_).

icsrCheck(proposition,np,X) :-
	!,
	\+ X has_intrinsic_feature inoCasemark(+).
icsrCheck(_,_,_).


ensureWhComp(CP) :-				 
	C head_of CP,
	(ec(C) 
	-> make C have_feature wh(+)		  % Q
	;  C has_feature wh(+)).

optWhComp(CP) :-  
	C head_of CP,
	make C have_feature wh(_) if ec(C).

% allow search for feature allowExt(Role) to be transparent
% wrt. VP subcategorization

vpAllowExt(Y,VP) :-
    (cat(VP,vp) ; cat(VP,ap)),
    (VP has_feature allowExt(X)
     -> X = Y
     ;  VP subcategorizes_for VPp,
	cat(VPp,vp), 
	vpAllowExt(Y,VPp)).

% allow search for feature allowExt(Role) to be transparent
% wrt. VP subcategorization

vpAllowExtL(L,VP) :-
	(VP has_feature allowExt(X)
	-> memberchk(X,L)
	;  VP subcategorizes_for VPp,
	   cat(VPp,vp), 
	   vpAllowExtL(L,VPp)) if cat(VP,vp).

vpRequiresExt(VP) :-
	(VP has_feature requiresExt(X)
	-> Y adjunct_of VP,
	   Y has_feature predicate(X),
	   !
	;  ((VP subcategorizes_for VPp, cat(VPp,vp))
	   -> vpRequiresExt(VPp,VP)
	   ;  true)) if cat(VP,vp).

vpRequiresExt(VP,MaxVP) :-
	VP has_feature requiresExt(X)
	-> Y adjunct_of MaxVP,
	   Y has_feature predicate(X),
	   !
	;  ((VP subcategorizes_for VPp, cat(VPp,vp))
	   -> vpRequiresExt(VPp,MaxVP)
	   ;  true).

%%
roleNotPresent(VP,Role) :-
	cat(VP,vp),
	(VP has_feature grid(Rs1,Rs2)
	-> \+ (member(Role,Rs1), member(Role,Rs2))
	;  (roleNotPresent(VPp,Role) if cat(VPp,vp)) 
	   if VP subcategorizes_for VPp).
	   

