%%%  -*- Mode: PROLOG; Package: PROLOG-USER -*-

%%% SAMPLE JAPANESE LEXICON
%%%
%%% (c) 1991, 1992, 1993, 1994  Sandiway Fong, NEC Research Institute, Inc.
%%%

% Demo sentence
% neko-ga koroshita nezumi-ga tabeta tiizu-wa kusatte ita

caseFoldInput.

default_features(n,[grid([],[]),case(_),theta(_)]).
default_features(v,[index(_)]).

no superClass(_,_).
no blockContraction(_).

%%% EXPORT
%%%	term(C)				terminals
%%%	lexicon(Word,C,Fs)		Word has category label C 
%%%					and feature list Fs
%%%	probeLexicon(Word)		Word is in lexicon
%%%	vMorphToFs(Base,Form,Features)	TNS/AGR features
%%%	inf(Verb,Type)
%%%	relevant(C)			constraints imposed by markers 
%%%					apply to C
%%% REFERENCES
%%%	optWhComplement(X)		xbar
%%%	(list processing)		utilities

term(n). term(v). term(a). term(p). term(c). term(adv). term(det). term($).
term(nq).

lexicon(Word,C,Fs) :- lex(Word,C,Fs).		% directly available
lexicon(Form,v,Fs) :-				% non-base verb forms
	lex(Form,v,Base,F1),
	verbFeatures(Base,F2),
	append1(F1,F2,Fs).

% deterministic
probeLexicon(Word) :- lex(Word,_,_) -> true ; lex(Word,_,_,_), !.

% :- dynamic edrJap:jlookup/4.

% lexicon(Word,C,Fs) :-
%	nonvar(Word)
%	-> (probeLexicon(Word)
%	   -> builtin(Word,C,Fs)
%	   ;  edrJap:jlookup(Word,_,C,Fs))
%	;  builtin(Word,C,Fs).

% probeLexicon(Word) :- lex(Word,_,_) ; lex(Word,_,_,_).

% builtin(Word,C,Fs) :- lex(Word,C,Fs).		% directly available
% builtin(Form,v,Fs) :-				% non-base verb forms
%	lex(Form,v,Base,F1),
%	verbFeatures(Base,F2),
%	append1(F1,F2,Fs).

%% nouns

% Wh-nouns

lex(nani,n,[a(-),p(-),agr([[],[],n]),wh(+),g('なに')]). % what
lex(nan,n,[a(-),p(-),agr([[],[],n]),wh(+),g('なに')]). % what
lex(dare,n,[a(-),p(-),agr([[],[],[m,f]]),wh(+),g('誰')]). % who
lex(dareka,n,[a(-),p(-),agr([[],[],[m,f]]),op(+),g('誰か')]). % someone
lex(daremo,n,[a(-),p(-),agr([[],[],[m,f]]),op(+),g('誰も')]). % everyone
lex(doko,n,[a(-),p(-),agr([[],[],n]),wh(+),g('どこ')]). % where

% People
% chomsky
lex(chomsky,n,[a(-),p(-),agr([3,sg,m]),class(person),g('チョムスキー')]).
lex(hanako,n,[a(-),p(-),agr([3,sg,f]),class(person),g('花子')]). % mary
lex(biru,n,[a(-),p(-),agr([3,sg,m]),class(person),g('ビル')]). % bill
lex(bill,n,[a(-),p(-),agr([3,sg,m]),class(person)]).
lex(masao,n,[a(-),p(-),agr([3,sg,f]),class(person),g('正雄')]). % male name (v. ambiguous kanji)
lex(mearii,n,[a(-),p(-),agr([3,sg,f]),class(person),g('メアリー')]). % mary
lex(mary,n,[a(-),p(-),agr([3,sg,f]),class(person)]). 
lex(taroo,n,[a(-),p(-),agr([3,sg,m]),class(person),g('太郎')]). % john
lex(john,n,[a(-),p(-),agr([3,sg,m]),class(person)]).

% Pronouns and Anaphors

lex(kare,n,[agr([3,sg,m]),notVar,g('彼'),goal(a(A),invPlusMinus(A,P)),p(P)]).

lex(kimi,n,[a(-),p(+),agr([2,sg,[m,f]]),g('君')]). % you
lex(karera,n,[a(-),p(+),agr([3,pl,[m,f]]),g('彼ら')]). % they

lex(soitsu,n,[a(-),p(+),agr([[],[],m]),g('そいつ')]). % (that) guy 
lex(soitsu,n,[a(+),p(-),agr([[],[],m]),g('そいつ')]). 
lex(watashi,n,[a(-),p(+),agr([1,sg,[m,f]]),g('私')]). % I

lex(otagai,n,[a(+),p(-),agr([3,pl,[m,f]]),g('お互い')]). 
% each other
lex(zibunzisin,n,[a(+),p(-),agr([3,sg,[m,f]]),g('自分自身')]).
% him/herself

% Nouns taking objects
% time (about)
lex(koro,n,[a(-),p(-),agr([[],[],n]),grid([],[reducedProp]),g('頃')]). 
% fact (hiragana: a4b3a4c8)
lex(koto,n,[a(-),p(-),agr([[],[],n]),grid([],[reducedProp]),g('こと')]).

% Common nouns
lex(kore,n,[a(-),p(-),agr([[],[],n]),g('これ')]). % this one
lex(sore,n,[a(-),p(-),agr([[],[],n]),g('それ')]). % that one (1)
lex(are,n,[a(-),p(-),agr([[],[],n]),g('あれ')]). % that one (2)
lex(aisukuriimu,n,[a(-),p(-),agr([[],[],n]),
	g('アイスクリーム')]). % ice-cream
lex(barriers,n,[a(-),p(-),agr([[],[],n]),grid([possessor],[]),
		g('「バリアズ」')]).  % barriers
lex(gengogaku,n,[a(-),p(-),agr([[],[],n]),
		 g('言語学')]). % linguistics
lex(gakusei,n,[a(-),p(-),agr([[],[],[m,f]]),
	       class(person),g('学生')]). % student
lex(gakkoo,n,[a(-),p(-),agr([[],[],n]),g('学校')]). % school
lex(gohan,n,[a(-),p(-),agr([[],[],n]),g('御飯')]). % boiled rice, meal
lex(haha,n,[a(-),p(-),agr([3,sg,f]),grid([possessor],[]),
	    class(person),g('母')]). % mother
lex(hahaoya,n,[a(-),p(-),agr([3,sg,f]),grid([possessor],[]),
 	       class(person),g('母親')]). % mother
lex(hito,n,[a(-),p(-),agr([[],[],[m,f]]),
	    class(person),g('人')]). % man
lex(hon,n,[a(-),p(-),agr([[],[],n]),
	   class(volume),g('本')]). % book
lex(kami,n,[a(-),p(-),agr([[],[],n]),g('髪')]). % hair
lex(kubi,n,[a(-),p(-),agr([[],[],n]),g('首')]). % neck/throat
lex(menyuu,n,[a(-),p(-),agr([[],[],n]),
	      g('メニュー')]).  % menu
lex(mura,n,[a(-),p(-),agr([[],[],n]),g('村')]).  % village
lex(neko,n,[a(-),p(-),agr([[],[],n]),g('猫')]). % cat
lex(okaasan,n,[a(-),p(-),agr([[],[],f]),grid([possessor],[]),
	       class(person),g('お母さん')]). % mother
lex(okane,n,[a(-),p(-),agr([[],[],n]),g('お金')]). % money
lex(nezumi,n,[a(-),p(-),agr([[],[],n]),g('鼠')]). % rat
lex(piza,n,[a(-),p(-),agr([[],[],n]),g('ピザ')]). % pizza
lex(sensei,n,[a(-),p(-),agr([[],[],[m,f]]),grid([possessor],[]),
	      class(person),g('先生')]). % teacher
lex(roka,n,[a(-),p(-),agr([]),grid([],[])]).	  % Roka
lex(sake,n,[a(-),p(-),agr([[],[],n]),class(hon),
	    g('酒')]).	  % sake/alcohol
lex(teeburu,n,[a(-),p(-),agr([[],[],n]),
	     g('テーブル')]). % table
lex(te,n,[a(-),p(-),agr([[],[],n]),g('手')]). % hand
lex(tegami,n,[a(-),p(-),agr([[],[],n]),g('手紙')]). % letter

lex(tiizu,n,[a(-),p(-),agr([[],[],n]),g('チーズ')]). % cheese

lex(ue,n,[a(-),p(-),agr([[],[],n]),grid([theme],[]),g('上')]). % top/on

% determiners
lex(kono,det,[g('この')]).	% this
lex(sono,det,[g('その')]).	% that (1)
lex(ano,det,[g('あの')]).	% that (2)
lex(dono,det,[wh(+),g('どの')]).	% which

%% verbs
%% stem forms - inconsistent

lex(itta,v,[allowExt(goal),
	    oneof(_,[[grid([agent],[]),morph(ik,past(+)),g('行った'),eng(go)],
		     [grid([agent],[proposition1]),morph(i,past(+)),g('言った'),eng(say)]])]).

% give s.t. (to s.o.)
lex(ager,v,[morph(ager,[]),grid([agent],[theme]),allowExt(goal)]).
lex(aisur,v,[morph(aisur,[]),grid([agent],[patient])]).
lex(a,v,[morph(a,[]),grid([agent],[]),allowExt(goal)]). % meet (with)
lex(hihans,v,[morph(hihans,[]),grid([agent],[patient])]). % criticize
lex(d,v,[morph(d,[]),aux,subcat(ap$[grid(_,_)],[])]). % copula
lex(ir,v,[morph(ir,[]),aux,prog,subcat(vp$[morph(_,te)],[])]). % be (prog.)
% get s.t. (using s.t.)
lex(irer,v,[morph(irer,[]),grid([agent],[themeNA]),allowExt(goal)]).
% say s.t.(to s.o.)
lex(i,v,[morph(i,[]),grid([agent],[proposition1]),allowExt(goal),eng(say)]). 
lex(ik,v,[morph(ik,[]),grid([agent],[]),allowExt(goal),eng(go)]). % go
lex(kak,v,[morph(kak,[]),grid([agent],[themeNA]),eng(write)]). % write
lex(kar,v,[morph(kar,[]),grid([agent],[theme]),eng(keep)]). % keep
lex(kau,v,[morph(kau,[]),grid([agent],[theme]),eng(buy)]). % buy
lex(kirawa,v,[morph(kirau,neg),grid([agent],[theme]),neg,g('嫌わ')]). % dislike
lex(kur,v,[morph(kur,[]),subcat(vp$[morph(_,te)],[])]). % come (conj).
lex(koru,v,[morph(koru,[]),grid([agent],[theme])]). % kill
lex(kusar,v,[morph(kusar,[]),grid([theme],[])]). % rotten
lex(mir,v,[morph(mir,[]),grid([agent],[theme])]). % see
lex(mora,v,[morph(mora,[]),grid([recipient],[theme]),allowExt(source)]). % receive
lex(mots,v,[morph(mots,[]),grid([agent],[theme])]). % have
lex(nagur,v,[morph(nagur,[]),grid([agent],[patient])]). % beat
lex(nar,v,[morph(nar,[]),grid([experiencer],[goal]),doCase(dat)]). % become
lex(ner,v,[morph(ner,[]),grid([theme],[])]).	  % sleep
lex(nemur,v,[morph(nemur,[]),grid([theme],[])]).  % to go to bed
lex(nom,v,[morph(nom,[]),grid([agent],[themeNA])]). % drink s.t.
lex(oi,v,[morph(oi,[]),grid([agent],[theme]),allowExt(goal)]). % put
% be angry about s.t.
lex(okor,v,[morph(okor,[]),grid([experiencer],[proposition])]). 
%lex(okur,v,[morph(okur,[]),grid([agent],[patient]),allowExt(goal)]). % send
lex(okur,v,[morph(okur,[]),grid([agent],[patient,goal]),idoCase(dat)]). % send
lex(omo,v,[morph(omo,[]),grid([agent],[proposition])]).	% think
lex(rare,v,[morph(rareru,[]),subcat(vp$[morph(_,[])],[noCasemark(+)]),
	    passive]). % passive 
lex(re,v,[morph(rareru,[]),subcat(vp$[morph(_,neg)],[noCasemark(+)]),
	  passive]).	% passive 
lex(sagas,v,[morph(sagas,[]),grid([agent],[theme])]). % search for s.t.
lex(shir,v,[morph(shir,[]),grid([theme],[proposition]),g('知')]).  % know
lex(suisen,v,[morph(suisen,[]),grid([agent],[theme])]).    % recommend
lex(sukinina,v,[morph(sukinina,[]),grid([agent],[patient])]). % fall in love with
%lex(su,v,[morph(su,[]),grid([agent],[]),allowExt(goal)]). % reside s.w.
lex(su,v,[morph(su,[]),grid([agent],[goal]),doCase(dat),g('住')]). % reside s.w.
lex(shookai,v,[morph(shookai,[]),grid([agent],[patient,goal]),idoCase(dat),g('紹介')]). % introduce s.o
lex(tabe,v,[morph(taberu,[]),grid([agent],[theme]),g('食')]). % eat
%lex(watas,v,[morph(watas,[]),grid([agent],[theme]),allowExt(goal)]). % give st.
lex(watas,v,[morph(watas,[]),grid([agent],[theme,goal]),idoCase(dat)]). % give st.
lex(yom,v,[morph(yom,[]),grid([agent],[themeNA])]). % read (s.t.)

% complementizers
lex(ka,c,[wh(+),yn,g('か')]).			% Q
lex(ka_doo_ka,c,[wh(+),merge(yn),g('かどうか')]).
						% Q - yes/no (whether or not)
lex(no,c,[wh(+),yn,g('の'),left(v,[],[])]).	% only in matrix clause
lex(to,c,[wh(-),selR([not(feature(inf(_)))]),g('と')]).	  % that
lex(tte,c,[wh(-),g('って')]).			% that

% adverbs
lex(doo,adv,[adjoin(left),predicate(manner),wh(+),g('どう')]).	% how
lex(itsu,adv,[adjoin(left),predicate(time),wh(+),g('いつ')]).   % when
lex(naze,adv,[adjoin(left),predicate(reason),wh(+),g('なぜ')]). 	
						  % why (kanji b2bfb8ce)
lex(kinoo,adv,[adjoin(left),predicate(time),g('昨日')]). % yesterday

% V-adj
lex(hitome,adv,[degree,g(b0ecccdc)]).		  % one glance
lex(sonnani,adv,[degree,g('そんなに')]).	  % so much
lex(mada,adv,[degree,g('まだ')]).		  % +/- still/yet

% adjectives
lex(nagai,a,[grid([theme],[]),g('長い')]).  % long (kanji c4b9a4a4)
lex(shizuka,a,[grid([theme],[]),g('静か')]).	  % quiet (na)
lex(genki,a,[grid([theme],[]),g('元気')]).	  % well
lex(tensai,a,[grid([theme],[]),g('天才')]).	  % genius

% postpositions

lex(de,p,[grid([],[location]),transparent,predicate(location),g('で')]).  % at

lex(kara,p,[grid([],[source]),transparent,predicate(source),g('から'),	% from
	  adjR([goal(vpAllowExt(source,X),X)])]).

% NB. Goal (.=. Location) (or Time - not added yet)
lex(ni,p,[grid([],[role(Role)]),transparent,predicate(Role),g('に'),
	  adjR([goal(externalRolesForNi(X,Role),X)])]).

lex(maeni,p,[grid([],[reducedProp]),predicate(time),g('前に')]). % before
lex(to,p,[grid([],role(Role)),conj,g('と'),adjR([feature(theta(Role))])]). % and

% markers
lex(ta,mrkr,[left(v,morph(_,past(+)),[]),g('た')]).
lex(wa,mrkr,[left(n,[],morphC(topic)),g('は')]).
lex(ga,mrkr,[left(n,[],morphC(nom)),g('が')]).
lex(o,mrkr,[left(n,[],morphC(acc)),g('を')]).
lex(wo,mrkr,[left(n,[],morphC(acc)),g('を')]).	  % orthography
lex(no,mrkr,[left(n,[],morphC(gen)),g('の')]).
lex(ni,mrkr,[left(n,[],morphC(dat)),g('に')]).


% NQs

% default
lex(hitotsu,nq,[number(1),classifier(default),g(a3b1a4c4)]).
lex(futatsu,nq,[number(2),classifier(default),g(a3b2a4c4)]).
lex(mittsu,nq,[number(3),classifier(default),g(a3b3a4c4)]).
lex(yottsu,nq,[number(4),classifier(default),g(a3b4a4c4)]).
lex(itsutsu,nq,[number(5),classifier(default),g(a3b5a4c4)]).
lex(muttsu,nq,[number(6),classifier(default),g(a3b6a4c4)]).
lex(nanatsu,nq,[number(7),classifier(default),g(a3b7a4c4)]).
lex(yattsu,nq,[number(8),classifier(default),g(a3b8a4c4)]).
lex(kokonotsu,nq,[number(9),classifier(default),g(a3b9a4c4)]).
lex(too,nq,[number(10),classifier(default),g(a3ba)]).

% person
lex(hitori,nq,[number(1),classifier(person),g(a3b1bfcd)]).
lex(futari,nq,[number(2),classifier(person),g(a3b2bfcd)]).
lex(sannin,nq,[number(3),classifier(person),g(a3b3bfcd)]).
lex(yonin,nq,[number(4),classifier(person),g(a3b4bfcd)]).
lex(gonin,nq,[number(5),classifier(person),g(a3b5bfcd)]).
lex(rokunin,nq,[number(6),classifier(person),g(a3b6bfcd)]).
lex(shichinin,nq,[number(7),classifier(person),g(a3b6bfcd)]).
lex(nananin,nq,[number(7),classifier(person),g(a3b7bfcd)]).
lex(hachinin,nq,[number(8),classifier(person),g(a3b8bfcd)]).
lex(kyuunin,nq,[number(9),classifier(person),g(a3b9bfcd)]).
lex(kunin,nq,[number(9),classifier(person),g(a3b9bfcd)]).
lex(juunin,nq,[number(10),classifier(person),g(a3babfcd)]).

% volume
lex(issatsu,nq,[number(1),classifier(volume),g(a3b1bafd)]).
lex(nisatsu,nq,[number(2),classifier(volume),g(a3b2bafd)]).
lex(sansatsu,nq,[number(3),classifier(volume),g(a3b3bafd)]).
lex(yonsatsu,nq,[number(4),classifier(volume),g(a3b4bafd)]).
lex(gosatsu,nq,[number(5),classifier(volume),g(a3b5bafd)]).
lex(rokusatsu,nq,[number(6),classifier(volume),g(a3b6bafd)]).
lex(shichisatsu,nq,[number(7),classifier(volume),g(a3b6bafd)]).
lex(nanasatsu,nq,[number(7),classifier(volume),g(a3b7bafd)]).
lex(hassatsu,nq,[number(8),classifier(volume),g(a3b8bafd)]).
lex(kyuusatsu,nq,[number(9),classifier(volume),g(a3b9bafd)]).
lex(jussatsu,nq,[number(10),classifier(volume),g(a3babafd)]).

% long object
lex(ippon,nq,[number(1),classifier(hon),g(a3b1cbdc)]).
lex(nihon,nq,[number(2),classifier(hon),g(a3b2cbdc)]).
lex(sanbon,nq,[number(3),classifier(hon),g(a3b3cbdc)]).
lex(yonhon,nq,[number(4),classifier(hon),g(a3b4cbdc)]).
lex(gohon,nq,[number(5),classifier(hon),g(a3b5cbdc)]).
lex(roppon,nq,[number(6),classifier(hon),g(a3b6cbdc)]).
lex(shichihon,nq,[number(7),classifier(hon),g(a3b6cbdc)]).
lex(nanahon,nq,[number(7),classifier(hon),g(a3b7cbdc)]).
lex(hachihon,nq,[number(8),classifier(hon),g(a3b8cbdc)]).
lex(happon,nq,[number(8),classifier(hon),g(a3b8cbdc)]).
lex(kyuhon,nq,[number(9),classifier(hon),g(a3b9cbdc)]).
lex(juppon,nq,[number(10),classifier(hon),g(a3bacbdc)]).


lexFeature(morphC(_),n).

% relevant for marker constraints
relevant(n).  relevant(v). relevant(p).

% non-base forms

lex(ageta,v,ager,[morph(ager,past(+)),g('あげた')]).
lex(aishite,v,aisur,[morph(aisur,te),g('愛して')]).

lex(au,v,a,[morph(a,past(-)),g('会う')]).
lex(atta,v,a,[morph(a,past(+)),g('会った')]).
lex(aitagatte,v,a,[morph(a,te),aux(signs(want)),g('会いたかった')]).

lex(da,v,d,[morph(d,past(-)),g('だ')]).
lex(desu,v,d,[morph(d,past(-)),formal,g('です')]).
lex(datta,v,d,[morph(d,past(+)),g('だった')]).
lex(deshita,v,d,[morph(d,past(+)),formal,g('でした')]).

lex(hihanshita,v,hihans,[morph(hihans,past(+)),g('批判した')]).
lex(ireta,v,irer,[morph(irer,past(+)),g('入れた')]).

lex(iru,v,ir,[morph(ir,past(-)),g('いる')]).
lex(ita,v,ir,[morph(ir,past(+)),g('いた')]).
lex(inai,v,ir,[morph(ir,npast(-)),neg,g('いない')]).

lex(iku,v,ik,[morph(ik,past(+)),g('行く')]).
% lex(itta,v,ik,[morph(ik,past(+)),g('行った')]).

lex(iu,v,i,[morph(i,past(-)),g('言う')]).
% lex(itta,v,i,[morph(i,past(+)),g('言った')]).

lex(kaita,v,kak,[morph(kak,past(+)),g('書いた')]).

lex(katta,v,kau,[morph(kau,past(+)),g('買った')]).

lex(katte,v,kar,[morph(kar,te),g('飼って')]).

lex(kite,v,kur,[morph(kur,te),g('来て')]).

lex(kuru,v,kur,[morph(kur,past(-)),g('来る')]).

lex(kita,v,kur,[morph(kur,past(+)),g('来た')]).

lex(koroshita,v,koru,[morph(koru,past(+)),g('殺した')]).
lex(kusatte,v,kusar,[morph(kusar,te),g('腐って')]).

lex(mita,v,mir,[morph(mir,past(+)),g('見た')]).

lex(morau,v,mora,[morph(mora,past(-)),g('貰う')]).
lex(moratta,v,mora,[morph(mora,past(+)),g('貰った')]).

lex(motsu,v,mots,[morph(mots,past(-)),g('持つ')]).
lex(motte,v,mots,[morph(mots,te),g('持って')]).
lex(motta,v,mots,[morph(mots,past(+)),g('持った')]).

lex(naguru,v,nagur,[morph(nagur,past(-)),g('殴る')]).
lex(nagutta,v,nagur,[morph(nagur,past(+)),g('殴った')]).

lex(natta,v,nar,[morph(nar,past(+)),g('なった')]).

lex(nemuru,v,ner,[morph(nemur,past(-)),g('眠る')]).

lex(neru,v,ner,[morph(ner,past(-)),g('寝る')]).

lex(nonda,v,nom,[morph(nom,past(+)),g('飲んだ')]).
lex(nonde,v,nom,[morph(nom,te),g('飲んで')]).
lex(nomimashita,v,nom,[morph(nom,past(+)),g('飲みました')]).

lex(oita,v,oi,[morph(oi,past(+)),g('置いた')]).

lex(okotte,v,okor,[morph(okor,te),g('怒って')]).

lex(okuru,v,okur,[morph(okur,past(-)),g('送る')]).
lex(okutta,v,okur,[morph(okur,past(-)),g('送った')]).

lex(omou,v,omo,[morph(omou,past(-)),g('思う')]).
lex(omotta,v,omo,[morph(omou,past(+)),g('思った')]).
lex(omotte,v,omo,[morph(omou,te),g('思って')]).
lex(omoikonde,v,omo,[morph(omoikomu,te),g('思い込んで')]). 

lex(rareru,v,rare,[morph(rareru,past(-)),g('られる')]).
lex(rareta,v,rare,[morph(rareru,past(+)),g('らっれた')]).
lex(reru,v,re,[morph(rareru,past(-)),g('れる')]).
lex(reta,v,re,[morph(rareru,past(+)),g('れた')]).
lex(rete,v,re,[morph(rareru,te),g('れて')]).

lex(sagashite,v,sagas,[morph(sagas,te),g('探って')]).

lex(shitte,v,shir,[morph(shir,te),g('知って')]).
lex(shiranai,v,shir,[morph(shir,npast(-)),neg,g('知らない')]).

lex(shookaisuru,v,shookai,[morph(shookai,past(-)),g('紹介する')]).
lex(shookaishita,v,shookai,[morph(shookai,past(+)),g('紹介した')]).

lex(suisenshita,v,suisen,[morph(suisen,past(+)),g(bfe4c1a6a4b7a4bf)]).

lex(sukininaru,v,sukinina,[morph(sukinina,past(-)),g('好きになる')]).
lex(sukininatta,v,sukinina,[morph(sukinina,past(+)),g('好きになった')]).

lex(sunde,v,su,[morph(su,te),g('住んで')]).
lex(sumu,v,su,[morph(su,past(-)),g('住む')]).

lex(taberu,v,tabe,[morph(taberu,past(-)),g('食べる')]).
lex(tabeta,v,tabe,[morph(taberu,past(+)),g('食べた')]).

lex(watashita,v,watas,[morph(watas,past(+)),g('渡した')]).

lex(yomu,v,yom,[morph(yom,past(-)),g('読む')]).
lex(yonda,v,yom,[morph(yom,past(+)),g('読んだ')]).
lex(yonde,v,yom,[morph(yom,te),g('読んで')]).
lex(yomimashita,v,yom,[morph(yom,past(+)),g('読みました')]).

% CONTRACTIONS

contraction([],X+iru,[X=morph(_,te),iru]).
contraction([],X+rareta,[X=morph(_,[]),rareta]).
contraction([],X+rareru,[X=morph(_,[]),rareru]).
contraction([],X+reta,[X=morph(_,neg),reta]).
contraction([],X+rete,[X=morph(_,neg),rete]).

no contraction(_,_,_,_).

% morphological case spellout

spellout_morphC(topic,'は').
spellout_morphC(acc,'を').
spellout_morphC(nom,'が').
spellout_morphC(dat,'に').
spellout_morphC(gen,'の').

spellout_2ndline(X,SL) :- 
    X has_feature morphC(C), containsWord(X,W), format(atom(SL),'~w-~w',[W,C]).

spellout_3rdline(X,TL) :-
    X has_feature morphC(C), spellout_morphC(C,K), X has_feature g(W),
    format(atom(TL),'~w~w',[W,K]).

% MISCELLANEOUS

verbalize([Y|s]) :- lex(Y,n,_).

nounFeatures(Base,F) :-	lex(Base,n,F1),	pick1(agr(_),F1,F).
verbFeatures(Base,F) :- 
	lex(Base,v,F1), 
	pick1(morph(_,_),F1,F2),
	pickNF1(g(_),F2,F).

%% MAPS MORPHOLOGY INTO SYNTACTIC FEATURES
%%	Verb morphology and Agreement
%% 
%%	Form	Tense		AGR
%%	base 	infinitival
%%	te	
%%	past(+) past(+)		agr(_)
%%	past(-) past(-)		agr(_)

vMorphToFs(_,Form,Features) :-
	formToFeatures(Form,Features).

formToFeatures([],[]).
formToFeatures(neg,[]).
formToFeatures(te,[]).
formToFeatures(past(X),[past(X),agr(_)]).
formToFeatures(npast(X),[past(X),agr(_)]).
formToFeatures(prog,[prog,past(-),agr(_)]).

% verb morphology
% iru1 (exists) -> ite
% iru2 (prog) -> ite
% iu (say) -> itte
% iru3 (need) -> itte

no inf(_,_).

% Head movement and negation

negFromV(Neg,V) :-
	V has_feature neg,
	mkFs([index(_),polarity(-)],Fs),
	mkEC(neg,Fs,Neg).

