%%%  -*- Package: PROLOG-USER; Mode: PROLOG -*-

:- ensure_loaded(scramblingRT).

%%% PERIPHERY FOR JAPANESE
%%%
%%% (c) 1991, 1992, 1993  Sandiway Fong, NEC Research Institute, Inc.
%%%
%%% Language-particular operations + kludgey stuff
%%%	1. case agreement
%%%	2. constrained scrambling
%%%

%%% S-STRUCTURE GRAMMAR ADDITIONS

% Experimental feature pushing
pushFeature(morphC(_)).

rule ecNP     -> [np(NP)] st ec(NP).
rule opC2$c2  -> [ecNP,c1].

rule head_adjoined _ adjoins_to_the left.	  % in head movement

% Pushed features: Will be automatically generated...
 
rule dObjectNP  -> [np(NP)] st \+ C==nom if NP has_feature morphC(C).
rule ioObjectNP  -> [np(NP)] st C==dat if NP has_feature morphC(C).
rule overtONP -> [overtNP(NP)] st (C==acc;C==dat) if  NP has_feature morphC(C).
rule objectNP -> [np(NP)] st (C==acc;C==dat) if  NP has_feature morphC(C).
rule subjectNP -> [np(NP)] st \+ (C==acc ; C==dat) if NP has_feature morphC(C).
rule npSubjectNP -> [np(NP)] st C==gen if NP has_feature morphC(C).

adjunction rule vp -> [overtONP,vp].		  % object scrambling (VP-int)
adjunction rule i2 -> [overtONP,i2].		  % no intermediate traces

% Base adjunction
adjunction rule np -> [overtNP,nq].		  % freely adjoin NQ to NP
adjunction rule np -> [nq,np].
adjunction rule np -> [pp,np] st lexicalProperty(pp,conj).

rhs [overtONP(NP),vp] add_goals [aPos(NP)].	  % scramble object to A-pos
rhs [overtONP(NP),i2] add_goals [aPosC(NP)].	  % A-pos (tentatively)

rhs [vp(VP),v] add_goals [\+ adjoinedMax(VP)].	  % eliminate unnecessary
						  % non-determinism
% NQ NP Agreement
rhs [overtNP(NP),nq(NQ)] add_goals [agreeNPNQ(NP,NQ)]. % eliminate non-det.
rhs [nq(NQ),np(NP)] add_goals [agreeNPNQ(NP,NQ)].

% Scrambling 
lhs overtONP add_goals [pushReq(es(i),es(o))].
lhs dObjectNP & rhs [np(X)] add_goals [cReq(X,es(i),es(o))].
lhs ioObjectNP & rhs [np] add_goals [cReq(es(i),es(o))].
lhs subjectNP & rhs [np(X)] add_goals [ldReq(X,es(i))].
lhs leftvgridcsr1stnp add_goals [oneReq(es(i))].
lhs v0 add_goals [zeroReq(es(i))].

rhs [det,n1] add_inherit plus(2,[1,[wh(_),op(_)]]).

% rhs [pp,vp] replace_rhs [coPP,vp].
rhs [c2,relClNP] replace_rhs [opC2,relClNP].

% Experimental feature pushing, again...
rhs [np,vgrid] replace_rhs [dObjectNP,vgrid].	  % opt. direct object
rhs [np,v1] replace_rhs [ioObjectNP,v1].	  % opt. indirect object
rhs [np,i1] replace_rhs [subjectNP,i1].		  % opt. subject
rhs [np,pgrid] replace_rhs [overtNP,pgrid].	  % disallow post-pos stranding
rhs [np,n1] replace_rhs [npSubjectNP,n1].	  % genitive Case

% null C not permitted for argument CPs
lhs v1gridcsr1stc2 & rhs anywhere c2(X) app_goals [(\+ nullComp(X))].

left_bracket c2 substitute openReq for open.

%%% S-STRUCTURE GRAMMAR DELETIONS

%% kind of redundant, will not be needed in next version

block rule adv -> [adv(Adv)] st maybeSubcategorized(adv,Adv). 

%%% OTHER LANGUAGE-SPECIFIC AREAS

%%% EMPTY COMP

emptyCompFeatures(Fs) :- mkFs([wh(-)],Fs).

%%% Move-Alpha (D-structure to S-structure)

moves(CF,np) :- cat(CF,np).

% compatibleCase(AssignedCase,MorphologicallyRealizedCase)

compatibleCase(X,X).
compatibleCase(_,topic).		  % deal with topicalization later

% Case Transmission: Need it for scrambling (complement to adjunct)
% NB. need to do [NP NQ NP-t], despite extraction, NQ-NP is overt

caseTransmission(Hd,NP,Case) :- 
	baseTrace(NP),
	headOfChain(Head,NP),
	Head has_feature adjunct,		  % scrambling
	NP has_feature compl,
	assignSCase(Hd,Case,Head),
	NP has_feature case(Case) if \+ ec(NP).	  % [NP NQ NP-t]

% Indicate all non-adjunct Case are realized using overt markers.

realizedAsMarker(X) :- X \== obq.

caseRealizationMode(_NP,morphC).

% NQ NP agreement

agreeNPNQ(NP,NQ) :-
	NQ has_feature classifier(Class),
	((\+ ec(NP) ; NP has_feature class(_))
	-> agreeClassifier(NP,Class)
	;  NP has_feature ec(trace),		  % force trace
	   transmitViaChain([],[goal(agreeClassifier1(X,Class),X)],NP)).

agreeClassifier1(NP,Class) :- agreeClassifier(NP,Class).

agreeClassifier(NP,Class1) :-
	NP has_feature class(Class)
	-> Class = Class1
	;  Class1 = default.

%% Null Comp C2
nullComp(CP) :-
	C head_of CP,
	ec(C).

%% Chain Formation conditions

chainLinkConditions(Head,Trace,_,UpPath,DownPath) :-
	\+ vacuousScrambling(UpPath,DownPath) if adjunctAPos(Trace),
	longDistABarPos(Head,UpPath) if adjunctAPos(Head).

adjunctAPos(X) :- X has_feature apos, X has_feature adjunct.  % scrambling

vacuousScrambling([],_).			  % no topmost segment crossed
vacuousScrambling(_,Down) :- Down \== [].

% Long Distance scrambling is A-bar

% cancels default assumption that it is an A-position 
% (see call to aPos/2 above) 
longDistABarPos(Head,UpPath) :-
	addFeature(apos,Head) 
	if (cat(CP,c2), member(CP,UpPath)). % inter-clausal

%%% LEXICON SUPPORT

% Priority correct?

externalRolesForNi(X,Y) :-
	vpAllowExtL([goal,source],X)
	-> Y = goal
	;  unsaturatedExtRole(X,agent),
	   Y = agent.

% NB: should allow for [VP [V2 [V1 [V V Adv]]]] collapsing to [VP V Adv]
adjoinedMax(X) :- adjoined(X,_,Y), cat(Y,C), max(C).
