%% SCRAMBLING
%%
%%	Must prevent vacuous scrambling

shiftRequest(n,es).

% request carrier r(State)
% State = Var or 1

%% pushReq(ES,ES')	start new req state 0
%% shiftReq(ES)		all requests state 0 -> 1
%% cReq(ES,ES')		ticks off a state 1 req
%% cReq(X,ES,ES')	X must be ec if req found
%% openReq(ES,ES') 	put in place of open, barf if state 0 req found

% initiates a request
pushReq(ES,[r(_)|ES]) :- kReq(ES).

% handles r([X])
kReq([X|ES]) :-
	open(X) 
	-> true
	;  (functor(X,r,_)
	   -> kReq1(ES)
	   ;  kReq(ES)).

% fails if we get to r(_) before an open
kReq1([X|ES]) :- open(X) -> true ;  \+ functor(X,r,_), kReq1(ES).

% change state of all open requests
% handles r([X])

shiftReq([X|ES]) :-	   
	open(X)
	-> true
	;  ((X = r(1) ; X = r([1]))		  % state <- 1
	   -> shiftReq(ES)
	   ;  shiftReq(ES)).

ldReq(Item,ES) :- ec(Item) -> ldReq(ES) ; true.	   
	
ldReq([X|ES]) :-	   
	open(X)
	-> true
	;  (X = r(V)
	   -> (var(V) -> V = [_] ; true),
	      shiftReq(ES)
	   ;  shiftReq(ES)).

%  consume one shifted request

cReq(ES,ESp) :-
	ES = [X|ES1],
	(open(X)
	-> ESp = ES
	;  (X = r(S)
	   -> (S == 1				  % consume
	      -> ESp = ES1
	      ;  ESp = [X|ESp1],
	         cReq(ES1,ESp1))
	   ;  ESp = [X|ESp1],
	      cReq(ES1,ESp1))).


% obligatory consume shifted request

cReq(Item,ES,ESp) :-
	ES = [X|ES1],
	(open(X)
	-> ESp = ES
	;  (X = r(S)
	   -> (S == 1				  % shifted
	      -> withEmpty(Item),
	         ESp = ES1
	      ;  ESp = [X|ESp1],
	         cReq(Item,ES1,ESp1))
	   ;  ESp = [X|ESp1],
	      cReq(Item,ES1,ESp1))).

withEmpty(X) :- ec(X) -> true ; adjoined(X,_,X1), withEmpty(X1).

% non-local request propagation
% ES = [...Rs...]
% ES' = [..Rs...,open,...]
% Translates r([1]) -> r(1)

openReq(ES,ESp) :-
	nlReq1(ES,ES1,Rs),
	append1(Rs,ES2,ESp),
	open(ES1,ES2).

% separates local requests Rs leaving ES'
nlReq1([],[],[]).
nlReq1([X|ES],ESp,Rs) :-
	open(X)
	-> ESp = [X|ES],
	   Rs = []
	;  (X = r(S)
	   -> (S == 1				  % already shifted
	      -> Rs = [X|Rsp],
	         nlReq1(ES,ESp,Rsp)
	      ;  S == [1],			  % shift, xform r([1])->r(1)
	         Rs = [r(1)|Rsp],
	         nlReq1(ES,ESp,Rsp))
	   ;  ESp = [X|ESp1],
	      nlReq1(ES,ESp1,Rs)).

% <= 1 state 1 req, no state 0 req
oneReq([X|ES]) :-
	open(X)
	-> true
	;  (X = r(S)
	   -> S == 1,
	      zeroReq(ES)
	   ;  oneReq(ES)).

% no reqs of any state allowed

zeroReq([X|ES]) :- open(X) -> true ;  \+ functor(X,r,_), zeroReq(ES).
